-- Test invoice_tax_amount
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(12);

set search_path to numerus, auth, public;

select has_view('invoice_tax_amount');
select table_privs_are('invoice_tax_amount', 'guest', array[]::text[]);
select table_privs_are('invoice_tax_amount', 'invoicer', array['SELECT']);
select table_privs_are('invoice_tax_amount', 'admin', array['SELECT']);
select table_privs_are('invoice_tax_amount', 'authenticator', array[]::text[]);

select has_column('invoice_tax_amount', 'invoice_id');
select col_type_is('invoice_tax_amount', 'invoice_id', 'integer');

select has_column('invoice_tax_amount', 'tax_id');
select col_type_is('invoice_tax_amount', 'tax_id', 'integer');

select has_column('invoice_tax_amount', 'amount');
select col_type_is('invoice_tax_amount', 'amount', 'integer');


set client_min_messages to warning;
truncate invoice_product_tax cascade;
truncate invoice_product cascade;
truncate invoice cascade;
truncate contact cascade;
truncate tax cascade;
truncate tax_class cascade;
truncate payment_method cascade;
truncate company cascade;
reset client_min_messages;

set constraints "company_default_payment_method_id_fkey" deferred;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country_code, currency_code, default_payment_method_id)
values (1, 'Company 1', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', 'ES', 'EUR', 111)
;

insert into payment_method (payment_method_id, company_id, name, instructions)
values (111, 1, 'cash', 'cash')
;

set constraints "company_default_payment_method_id_fkey" immediate;

insert into tax_class (tax_class_id, company_id, name)
values (11, 1, 'tax')
;

insert into tax (tax_id, company_id, tax_class_id, name, rate)
values (2, 1, 11, 'IRPF -15 %', -0.15)
     , (3, 1, 11, 'IVA 4 %', 0.04)
     , (4, 1, 11, 'IVA 10 %', 0.10)
     , (5, 1, 11, 'IVA 21 %', 0.21)
;

insert into contact (contact_id, company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country_code)
values (7, 1, 'Contact', 'XX555', '', '777-777-777', 'c@c', '', '', '', '', '', 'ES')
;

insert into invoice (invoice_id, company_id, invoice_number, invoice_date, contact_id, currency_code, payment_method_id)
values ( 8, 1, 'I1', current_date, 7, 'EUR', 111)
     , ( 9, 1, 'I2', current_date, 7, 'EUR', 111)
     , (10, 1, 'I3', current_date, 7, 'EUR', 111)
     , (11, 1, 'I4', current_date, 7, 'EUR', 111)
;

insert into invoice_product (invoice_product_id, invoice_id, name, price, quantity, discount_rate)
values (12,  8, 'P', 100, 1, 0.0)
     , (13,  8, 'P', 200, 2, 0.1)
     , (14,  9, 'P', 222, 3, 0.0)
     , (15,  9, 'P', 333, 4, 0.2)
     , (16, 10, 'P', 444, 5, 0.0)
     , (17, 10, 'P', 555, 6, 0.1)
     , (18, 11, 'P', 777, 8, 0.0)
;

insert into invoice_product_tax (invoice_product_id, tax_id, tax_rate)
values (12, 2, -0.15)
     , (12, 5,  0.21)
     , (13, 3,  0.04)
     , (14, 4,  0.10)
     , (14, 5,  0.21)
     , (14, 2, -0.07)
     , (15, 4,  0.10)
     , (16, 4,  0.10)
     , (16, 5,  0.21)
     , (17, 5,  0.21)
     , (17, 3,  0.04)
;   

select bag_eq(
	$$ select invoice_id, tax_id, amount from invoice_tax_amount $$,
	$$ values ( 8, 2,  -15)
	        , ( 8, 3,   14)
	        , ( 8, 5,   21)
	        , ( 9, 2,  -47)
	        , ( 9, 4,  174)
	        , ( 9, 5,  140)
	        , (10, 3,  120)
	        , (10, 4,  222)
	        , (10, 5, 1095)
	$$,
	'Should compute the amount for all taxes in the invoiced products.'
);


select *
from finish();

rollback;
