-- Test quote
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(76);

set search_path to numerus, auth, public;

select has_table('quote');
select has_pk('quote' );
select table_privs_are('quote', 'guest', array []::text[]);
select table_privs_are('quote', 'invoicer', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('quote', 'admin', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('quote', 'authenticator', array []::text[]);

select has_sequence('quote_quote_id_seq');
select sequence_privs_are('quote_quote_id_seq', 'guest', array[]::text[]);
select sequence_privs_are('quote_quote_id_seq', 'invoicer', array['USAGE']);
select sequence_privs_are('quote_quote_id_seq', 'admin', array['USAGE']);
select sequence_privs_are('quote_quote_id_seq', 'authenticator', array[]::text[]);

select has_column('quote', 'quote_id');
select col_is_pk('quote', 'quote_id');
select col_type_is('quote', 'quote_id', 'integer');
select col_not_null('quote', 'quote_id');
select col_has_default('quote', 'quote_id');
select col_default_is('quote', 'quote_id', 'nextval(''quote_quote_id_seq''::regclass)');

select has_column('quote', 'company_id');
select col_is_fk('quote', 'company_id');
select fk_ok('quote', 'company_id', 'company', 'company_id');
select col_type_is('quote', 'company_id', 'integer');
select col_not_null('quote', 'company_id');
select col_hasnt_default('quote', 'company_id');

select has_column('quote', 'slug');
select col_is_unique('quote', 'slug');
select col_type_is('quote', 'slug', 'uuid');
select col_not_null('quote', 'slug');
select col_has_default('quote', 'slug');
select col_default_is('quote', 'slug', 'gen_random_uuid()');

select has_column('quote', 'quote_number');
select col_type_is('quote', 'quote_number', 'text');
select col_not_null('quote', 'quote_number');
select col_hasnt_default('quote', 'quote_number');

select has_column('quote', 'quote_date');
select col_type_is('quote', 'quote_date', 'date');
select col_not_null('quote', 'quote_date');
select col_has_default('quote', 'quote_date');
select col_default_is('quote', 'quote_date', 'CURRENT_DATE');

select has_column('quote', 'quote_status');
select col_is_fk('quote', 'quote_status');
select fk_ok('quote', 'quote_status', 'quote_status', 'quote_status');
select col_type_is('quote', 'quote_status', 'text');
select col_not_null('quote', 'quote_status');
select col_has_default('quote', 'quote_status');
select col_default_is('quote', 'quote_status', 'created');

select has_column('quote', 'terms_and_conditions');
select col_type_is('quote', 'terms_and_conditions', 'text');
select col_not_null('quote', 'terms_and_conditions');
select col_has_default('quote', 'terms_and_conditions');
select col_default_is('quote', 'terms_and_conditions', '');

select has_column('quote', 'notes');
select col_type_is('quote', 'notes', 'text');
select col_not_null('quote', 'notes');
select col_has_default('quote', 'notes');
select col_default_is('quote', 'notes', '');

select has_column('quote', 'tags');
select col_type_is('quote', 'tags', 'tag_name[]');
select col_not_null('quote', 'tags');
select col_has_default('quote', 'tags');
select col_default_is('quote', 'tags', '{}');

select has_column('quote', 'currency_code');
select col_is_fk('quote', 'currency_code');
select fk_ok('quote', 'currency_code', 'currency', 'currency_code');
select col_type_is('quote', 'currency_code', 'text');
select col_not_null('quote', 'currency_code');
select col_hasnt_default('quote', 'currency_code');

select has_column('quote', 'created_at');
select col_type_is('quote', 'created_at', 'timestamp with time zone');
select col_not_null('quote', 'created_at');
select col_has_default('quote', 'created_at');
select col_default_is('quote', 'created_at', 'CURRENT_TIMESTAMP');


set client_min_messages to warning;
truncate quote cascade;
truncate company_user cascade;
truncate company cascade;
truncate payment_method cascade;
truncate auth."user" cascade;
reset client_min_messages;

insert into auth."user" (user_id, email, name, password, role, cookie, cookie_expires_at)
values (1, 'demo@tandem.blog', 'Demo', 'test', 'invoicer', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e', current_timestamp + interval '1 month')
     , (5, 'admin@tandem.blog', 'Demo', 'test', 'admin', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524', current_timestamp + interval '1 month')
;

set constraints "company_default_payment_method_id_fkey" deferred;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country_code, currency_code, default_payment_method_id)
values (2, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', 'ES', 'EUR', 222)
     , (4, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', 'FR', 'USD', 444)
;

insert into payment_method (payment_method_id, company_id, name, instructions)
values (444, 4, 'cash', 'cash')
     , (222, 2, 'cash', 'cash')
;

set constraints "company_default_payment_method_id_fkey" immediate;

insert into company_user (company_id, user_id)
values (2, 1)
     , (4, 5)
;

insert into quote (company_id, quote_number, currency_code)
values (2, 'QUO020001', 'EUR')
     , (4, 'QUO040001', 'EUR')
;


prepare quote_data as
select company_id, quote_number
from quote
order by company_id, quote_number;

set role invoicer;
select is_empty('quote_data', 'Should show no data when cookie is not set yet');
reset role;

select set_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog');
select bag_eq(
	'quote_data',
	$$ values (2, 'QUO020001')
	$$,
	'Should only list quotations from the companies where demo@tandem.blog is user of'
);
reset role;

select set_cookie('12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524/admin@tandem.blog');
select bag_eq(
	'quote_data',
	$$ values (4, 'QUO040001')
	$$,
	'Should only list quotations from the companies where admin@tandem.blog is user of'
);
reset role;

select set_cookie('not-a-cookie');
select throws_ok(
	'quote_data',
	'42501', 'permission denied for table quote',
	'Should not allow select to guest users'
);
reset role;


select throws_ok( $$
	insert into quote (company_id, quote_number, currency_code)
	values (2, '  ', 'EUR')
	$$,
       '23514', 'new row for relation "quote" violates check constraint "quote_number_not_empty"',
	'Should not allow quotes with a blank number'
);

select *
from finish();

rollback;

